package routing.util;

import core.*;

import java.util.Random;
public class EnergyModel implements ModuleCommunicationListener {
   public static final String INIT_ENERGY_S = "initialEnergy";
    public static final String SCAN_ENERGY_S = "scanEnergy";
    public static final String SCAN_RSP_ENERGY_S = "scanResponseEnergy";
    public static final String TRANSMIT_ENERGY_S = "transmitEnergy";
    public static final String WARMUP_S = "energyWarmup";
    public static final String ENERGY_VALUE_ID = "Energy.value";
    private final double[] initEnergy;
    private double warmupTime;
   private double currentEnergy;
   private double scanEnergy;
    private double transmitEnergy;
    private double scanResponseEnergy;
    private double lastUpdate;
    private ModuleCommunicationBus comBus;
    private static Random rng = null;
 public EnergyModel(Settings s) {
        this.initEnergy = s.getCsvDoubles(INIT_ENERGY_S);
        if (this.initEnergy.length != 1 && this.initEnergy.length != 2) {
            throw new SettingsError(INIT_ENERGY_S + " setting must have " +
                    "either a single value or two comma separated values");
        }
        this.scanEnergy = s.getDouble(SCAN_ENERGY_S);
        this.transmitEnergy = s.getDouble(TRANSMIT_ENERGY_S);
        this.scanResponseEnergy = s.getDouble(SCAN_RSP_ENERGY_S);
        if (s.contains(WARMUP_S)) {
            this.warmupTime = s.getInt(WARMUP_S);
            if (this.warmupTime == -1) {
                this.warmupTime = new Settings(report.Report.REPORT_NS).
                        getInt(report.Report.WARMUP_S);
            }
        } else {
            this.warmupTime = 0;
        }
    }
    protected EnergyModel(EnergyModel proto) {
        this.initEnergy = proto.initEnergy;
        setEnergy(this.initEnergy);
        this.scanEnergy = proto.scanEnergy;
        this.transmitEnergy = proto.transmitEnergy;
        this.warmupTime = proto.warmupTime;
        this.scanResponseEnergy = proto.scanResponseEnergy;
        this.comBus = null;
        this.lastUpdate = 0;
    }

    public EnergyModel replicate() {
        return new EnergyModel(this);
    }
    protected void setEnergy(double range[]) {
        if (range.length == 1) {
            this.currentEnergy = range[0];
        } else {
            if (rng == null) {
                rng = new Random((int) (range[0] + range[1]));
            }
            this.currentEnergy = range[0] +
                    rng.nextDouble() * (range[1] - range[0]);
        }
    }
    public double getEnergy() {
        return this.currentEnergy;
    }
    protected void reduceEnergy(double amount) {
        if (SimClock.getTime() < this.warmupTime) {
            return;
        }

        if (comBus == null) {
            return; /* model not initialized (via update) yet */
        }

        if (amount >= this.currentEnergy) {
            comBus.updateProperty(ENERGY_VALUE_ID, 0.0);
        } else {
            comBus.updateDouble(ENERGY_VALUE_ID, -amount);
        }

    }
    public void reduceDiscoveryEnergy() {
        reduceEnergy(this.scanResponseEnergy);
    }
    public void update(NetworkInterface iface, ModuleCommunicationBus comBus) {
        double simTime = SimClock.getTime();
        double delta = simTime - this.lastUpdate;
        if (this.comBus == null) {
            this.comBus = comBus;
            this.comBus.addProperty(ENERGY_VALUE_ID, this.currentEnergy);
            this.comBus.subscribe(ENERGY_VALUE_ID, this);
        }
        if (simTime > this.lastUpdate && iface.isTransferring()) {
            reduceEnergy(delta * this.transmitEnergy);
        }
        this.lastUpdate = simTime;

        if (iface.isScanning()) {
            if (iface.getTransmitRange() > 0) {
                if (delta < 1) {
                    reduceEnergy(this.scanEnergy * delta);
                } else {
                    reduceEnergy(this.scanEnergy);
                }
            }
        }
    }
    public void moduleValueChanged(String key, Object newValue) {
        this.currentEnergy = (Double) newValue;
    }
}
